## R version 4.4.0, RStudio 2024.04.01 Build 748

library(dplyr) # version 1.1.4 (left_join)
library(readxl) # version 1.4.3 (read_excel)
library(writexl) #version 1.5.0 (write_xlsx)
library(reshape2) # version 1.4.4 (melt)
library(imputeTS) # version 3.3 (na_kalman)
library(DescTools) # version 0.99.54 (Dummy)

# Variables needed to create panels
## Two different lists of country names because the names differ between sources
rm(list=ls())
all_countries <- c("Austria", "Belgium", "Bulgaria", "Croatia", "Cyprus", "Czech Republic", "Denmark", "Estonia",
                   "Finland", "France", "Germany", "Greece", "Hungary", "Iceland", "Ireland", "Italy", "Latvia",
                   "Liechtenstein", "Lithuania", "Luxembourg", "Malta", "Netherlands", "Norway", "Poland",
                   "Portugal", "Romania", "Slovakia", "Slovenia", "Spain", "Sweden", "Switzerland")
all_countries2 <- c("Austria", "Belgium", "Bulgaria", "Croatia", "Cyprus", "Czechia", "Denmark", "Estonia",
                   "Finland", "France", "Germany", "Greece", "Hungary", "Iceland", "Ireland", "Italy", "Latvia",
                   "Liechtenstein", "Lithuania", "Luxembourg", "Malta", "Netherlands", "Norway", "Poland",
                   "Portugal", "Romania", "Slovak Republic", "Slovenia", "Spain", "Sweden", "Switzerland")
file_endings <- c("", "_act", "_inact")
file_start <- c("firms_cit_agent")

# Create basic panel for years 1990 to 2022, all countries
pan <- expand.grid(1990:2022, all_countries)
names(pan) <- c("year", "country")
# loop through all three files showing the number of firm formations per year
# (although we only use the total numbers)
for (j in file_endings) {
  pan$x <- as.vector(as.matrix(read.csv(paste0(file_start, j, ".csv")))[,2:32])
  colnames(pan)[which(names(pan) == "x")] <- paste0(file_start, j)
}

## get GDP Growth data (from World Bank website):
GDPgrowth <- as.data.frame(read_excel("GDP growth (WorldBank) 2023.xls", skip = 3))
# identify the line numbers in the GDP file needed for the countries. 
cind <- sapply(all_countries2, function(X) which(GDPgrowth[, "Country Name"] == X))

# read in ISO3 country codes and GDP growth numbers from World Bank table and
# append it to panel
country_codes <- GDPgrowth[cind, "Country Code"]
pan$iso_3 <- as.vector(outer(1:33, country_codes, function(X, Y) Y))
pan$GDP_growth <-as.vector(t(GDPgrowth[cind,35:67]))

## get GDP  data (also from World Bank website):
GDP <- as.data.frame(read_excel("GDP (WorldBank) 2023.xls", skip = 3))
cind <- sapply(all_countries2, function(X) which(GDP[, "Country Name"] == X))
# append GDP and log(GDP) to panel
pan$GDP  <- as.vector(t(GDP[cind,35:67]))
pan$l.GDP <- log(pan$GDP)

## get tax rates from Tax Foundation website
# Source:
#   https://taxfoundation.org/publications/corporate-tax-rates-around-the-world/
taxes <- as.data.frame(read_excel("Tax_Foundation 1980-2022-Corporate-Tax-Rates-Around-the-World-1.xlsx"))
#convert to long format
tax_long <- melt(data = taxes[taxes$iso_3 %in% pan$iso_3,],
     id.vars = c("iso_3"),
     measure.vars = as.character(1990:2022),
     variable.name = "year",
     value.name = "corptax")
tax_long$year <- as.numeric(as.character(tax_long$year))
# append taxes to panel
pan <- left_join(pan, tax_long, by = c("year", "iso_3"))

## Read in Rule of Law (from World Bank / Doing Business): 
ROL <- as.data.frame(read_excel("wgidataset_2023.xlsx", 
                                sheet = "RuleofLaw", skip = 13, na = "#N/A"))
# only need to the columns with the "estimate", not the other things
ROL <- ROL[, c(1:2, which(ROL[1, ] == "Estimate"))]
colnames(ROL)[1:2] <- c("country", "iso_3")
colnames(ROL)[3:25] <- substr(colnames(ROL)[3:25], 1, 4)
ROL <- ROL[-1, ] # remove country name to match by ISO code
ROL[ROL$iso_3 == "ROM",]$iso_3 <- "ROU" # Romania's iso_3 code is different here!
# convert to long format
ROL_long <- melt(data = ROL[ROL$iso_3 %in% pan$iso_3,],
                 id.vars = c("iso_3"),
                 measure.vars = as.character(c(1996, 1998, 2000, 2002:2020)),
                 variable.name = "year",
                 value.name = "rule_of_law")
ROL_long$year <- as.numeric(as.character(ROL_long$year))
# append rule of law score to panel
pan <- left_join(pan, ROL_long, by = c("year", "iso_3"))
# The data were collected only biannually from 1996 to 2000. 
# I take the previous year's values for inbetween years:
pan$rule_of_law[which(pan$year %in% seq(1997, 2001, 2))] <- pan$rule_of_law[which(pan$year %in% seq(1996, 2000, 2))]

##Create dummy variables for EU & EEA membership:
EU_member <- c(1995, 1958, 2007, 2013, 2004, 2004, 1973, 2004, 1995, 1958, 1958, 1981, 2004, 9999, 1973, 1958, 2004,
               9999, 2004, 1958, 2004, 1958, 9999, 2004, 1986, 2007, 2004, 2004, 1986, 1995, 9999)
EEA_member <- c(1994, 1958, 2007, 2013, 2004, 2004, 1973, 2004, 1994, 1958, 1958, 1981, 2004, 1994, 1973, 1958, 2004,
                1995, 2004, 1958, 2004, 1958, 1994, 2004, 1986, 2007, 2004, 2004, 1986, 1994, 9999)
# converting this into binary variable for the panel (is the country a member in a given year?)
pan$EU <- as.vector(outer(1990:2022, EU_member, function(X, Y) as.numeric(X >= Y)))
pan$EEA <- as.vector(outer(1990:2022, EEA_member, function(X, Y) as.numeric(X >= Y)))
pan$EU_EEA <- as.numeric(pan$EU | pan$EEA)
# create dummy for Centros case
pan$Centros <- as.numeric(as.numeric(as.character(pan$year)) > 1999)

##Add migrant data
mtab <- read_excel("migrantsUK2020.xlsx")
names(mtab)[6] <- "country" # change country column name so that it can be matched
mtab$country <- gsub("\\*", "", mtab$country) # remove asterisks from column
# convert to long format
migra_long <- melt(data = mtab[mtab$country %in% c(all_countries, all_countries2),],
                   id.vars = c("country"),
                   measure.vars = as.character(seq(1995, 2020, 5)),
                   variable.name = "year",
                   value.name = "migrants")
migra_long <- migra_long[order(migra_long$country), ] # sort by country so that it matched order in panel
migra_long$year <- as.numeric(as.character(migra_long$year))
migra_long$country[migra_long$country == "Czechia"] <- "Czech Republic"
# We match migration numbers with panel - now in a separate X file
# Then we interpolate missing numbers using Kalman smoothing
# Finally, we append migrants and log(migrants) to panel
X <- left_join(pan, migra_long, by = c("country", "year")) 
pan$migrants <- unlist(by(X$migrants, X$country, na_kalman))
pan$l.migrants <- log(pan$migrants)
rm(X) # remove unnecessary extra panel

# Total firms founded
totals <- as.data.frame(read_excel("all_firms_by_country_year_2023.xlsx", skip = 4))
names(totals)[1] <- "country"
## convert to long format:
totals_long <- melt(data = totals[sapply(all_countries, function(X) grep(X, totals$country)), ],
                 id.vars = c("country"),
                 measure.vars = as.character(c(1990:2021)),
                 variable.name = "year",
                 value.name = "total_firms_founded")
totals_long$country <- sub("\\s\\(.*", "", totals_long$country) # remove country abbreviation in parentheses
totals_long$year <- as.numeric(as.character(totals_long$year))
## append total firms to panel and add the UK based firms 
pan <- left_join(pan, totals_long, by = c("country", "year"))
pan$total_firms_founded <- as.numeric(pan$total_firms_founded)
pan$total_firms_plus <- pan$firms_cit_agent + pan$total_firms_founded

# create logs of number of UK-based firms and total firms
pan$lPD <- log(pan$firms_cit_agent + 1)
pan$l.totalfirms <- log(pan$total_firms_plus + 1)

# load  minimum capital numbers and append to panel
mincap2023 <- read_excel("mincap_2023.xlsx")
pan <- left_join(pan, mincap2023[, c("year", "country", 
                    "mincap.USD", "minpayin.USD", "GDP.PC.PPP", "mincap.PP",
                    "minpayin.PP", "l.mincap.PP", "l.minpayin.PP")], 
                          by = c("year", "country"))

# add treatment variables for different "levels" of minimum capital requirements
refs2 <- read_excel("reforms_coding_2023.xlsx")
refs2 <- cbind(refs2, Dummy(refs2$Category))
pan <- left_join(pan, refs2[, c("year", "country", "Category")], by = c("country", "year"))
names(pan)[ncol(pan)] <- "Reform"

write_xlsx(pan, "panel_7_2023.xlsx")