library(readxl) # version 1.4.3 (read_excel)
library(xtable) # version 1.8-4 (xtable; also needed in corstars)
library(Hmisc)  # version 5.1-2 (rcorr; needed in corstars)
library(DescTools) # version 0.99.54 (Dummy; needed in corstars)
library(gt) # Version 0.10.1 (gtsave)
library(gtsummary) # version 1.7.2 (table_summary)
library(ggplot2) # version 3.5.1. (ggplot)
library(scales) #version 1.3.0 (scales for ggplot)
library(cowplot) #version 1.1.3 (get_plot_component, plot_grid)

dp <- read_excel("panel_7_2023.xlsx")
dp <- as.data.frame(subset(dp, year %in% 1995:2020))
all_countries <- c("Austria", "Belgium", "Bulgaria", "Croatia", "Cyprus", "Czech Republic", "Denmark", "Estonia",
                   "Finland", "France", "Germany", "Greece", "Hungary", "Iceland", "Ireland", "Italy", "Latvia",
                   "Liechtenstein", "Lithuania", "Luxembourg", "Malta", "Netherlands", "Norway", "Poland",
                   "Portugal", "Romania", "Slovakia", "Slovenia", "Spain", "Sweden", "Switzerland")

# some variables are read as character but are needed as factor or numeric
dp$country <- as.factor(dp$country)
dp$corptax <- as.numeric(dp$corptax)
dp$rule_of_law <- as.numeric(dp$rule_of_law)

# Table 1: Average number of cross-country incorporations for five year periods
## use integer division to put years in give-year groups then take average by country/year
groups <- (as.numeric(as.character(dp$year)) - 1991) %/% 5 
avgs <- by(dp$firms_cit_agent, list(groups, dp$country), mean, na.rm = TRUE)
## reformat "by" object into a data.frame and create rownames and colnames; 
## exclude 1995
time_table <- as.data.frame(t(matrix(as.vector(avgs), ncol = 31))[, -1])
rownames(time_table) <- levels(dp$country)
colnames(time_table) <- c("1996-2000", "2001-2005", "2006-2010", "2011-2015", "2015-2020")
## save as html
write.table(print(xtable(time_table), type = "html"), file = "time_table.html")
## The html file's content was subsequently copied into the paper and reformatted. 
## The countries were ordered by groups in Word.

# Table 4: Pearson correlations
## Define corstars as a function that creates a correlation table: 
### Parameters
### x is a matrix containing the data
### method: correlation method. "pearson"" or "spearman"" is supported
### removeTriangle : remove upper or lower triangle
### results:  Should it be saved as "html" or "latex" format?
corstars <- function(x, method=c("pearson", "spearman"), removeTriangle=c("upper", "lower"),
                    result=c("none", "html", "latex")){
  #Compute correlation matrix
  stopifnot(require(Hmisc))
  if (result[1] %in% c("html", "latex")) stopifnot(require(xtable))
  xclass <- sapply(x, class)
  dummies <- lapply(which(xclass %in% c("character", "factor")), function(X) DescTools::Dummy(x[, X]))
  if (length(dummies) > 0) 
  {
    x <- x[, -which(xclass %in% c("character", "factor"))]
    x <- cbind(x, do.call(cbind, dummies))
    warning("Characters and factors converted to dummies")
  }
  x <- as.matrix(x)
  correlation_matrix<-rcorr(x, type=method[1])
  R <- correlation_matrix$r # Matrix of correlation coefficients
  p <- correlation_matrix$P # Matrix of p-value 
  
  ## Define notions for significance levels; spacing is important for formatting.
  mystars <- ifelse(p < .01, "***", ifelse(p < .05, "** ", ifelse(p < .1, "*  ", "   ")))
  
  ## trunctuate the correlation matrix to two decimals
  R <- format(round(cbind(rep(-1.11, ncol(x)), R), 2))[,-1]
  
  ## remove NAs
  R[grep("NA", R)] <- ""
  R[grep("NA", R)] <- ""
  mystars[is.na(mystars)] <- ""
  
  ## build a new matrix that includes the correlations with their appropriate stars
  Rnew <- matrix(paste(R, mystars, sep=""), ncol=ncol(x))
  diag(Rnew) <- paste(diag(R), " ", sep="")
  rownames(Rnew) <- colnames(x)
  colnames(Rnew) <- paste(colnames(x), "", sep="")
  
  ## remove upper triangle of correlation matrix
  if(removeTriangle[1]=="upper"){
    Rnew <- as.matrix(Rnew)
    Rnew[upper.tri(Rnew, diag = TRUE)] <- ""
    Rnew <- as.data.frame(Rnew)
  } else if(removeTriangle[1]=="lower") ## remove lower triangle of correlation matrix
  {
    Rnew <- as.matrix(Rnew)
    Rnew[lower.tri(Rnew, diag = TRUE)] <- ""
    Rnew <- as.data.frame(Rnew)
  }
  
  ## remove last column and return the correlation matrix
  Rnew <- cbind(Rnew[1:length(Rnew)-1])
  if (result[1]=="none") 
    return(Rnew) else {
      if(result[1]=="html") print(xtable(Rnew), type="html") else
        print(xtable(Rnew), type="latex") 
    }
} 

## call corstars to create correlation table and save as html file (to copy into document)
desctab <- corstars(dp[, c("lPD", "l.GDP", "l.totalfirms", "l.migrants", "corptax","rule_of_law",
                           "l.mincap.PP", "l.minpayin.PP")], result = "html")
write.table(desctab, file = "corrtab.html")

# Table 5: Summary statistics by "level" of capital requirement
pan_table <- tbl_summary(data = dp[, c("Reform", "l.GDP", "l.totalfirms", 
                                         "l.migrants", "corptax", "rule_of_law")],
                         by = Reform,
                         label = list(
                           l.GDP ~"log(GDP)", 
                           l.totalfirms ~ "log(total no. firms)",
                           l.migrants ~ "log(migrants)", 
                           corptax ~ "corporate tax", 
                           rule_of_law ~ "Rule of Law"
                         ),
                         statistic = list(all_continuous() ~ "{mean} ({sd})"),
                         missing = "no"
  ) %>% add_overall() %>% modify_header(label = "**All**", 
                                      stat_1 = "**No (meaningful) minimum capital**, (N = {n})",
                                      stat_2 = "**Low minimum capital**, (N = {n})",
                                      stat_3 = "**High minimum capital**, (N = {n})") %>%
  modify_caption("**Table 5: Summary statistics by level of capital requirement**") 

gtsave(as_gt(pan_table), "panel_table.rtf")


#Online Appendix: Figure showing development of minimum capital and number of firms
plotdat <- as.data.frame(dp[,c("country", "year", "firms_cit_agent", "lPD", "mincap.PP", "minpayin.PP")])
plotdat <- subset(plotdat, year %in% 1995:2020)

# calculate scale for each country's graphs
plotdat$country_scale <- sapply(plotdat$country, function(X) max(subset(plotdat, country == X)$firms_cit_agent))
plotdat$scaled_firms <- 2 * plotdat$firms_cit_agent / plotdat$country_scale
country_scale <- sapply(all_countries, function(X) max(subset(plotdat, country == X)$firms_cit_agent))

# split the plot data into a list with one element for each country
plotdat2 <- split(plotdat, plotdat$country)

# function to draw the plot for one country each
my_plot <- function(data, xaxis = TRUE, yaxis_left = TRUE) {
  scale_factor <- country_scale[data$country[1]]
  country_name <- data$country[1]
  p <- ggplot(data, aes(x = year)) + 
    geom_line(aes(y = mincap.PP, linetype = "Minimum capital / GDP(PPP)")) +
    geom_line(aes(y = minpayin.PP, linetype = "Minimum pay-in / GDP(PPP)")) +
    geom_line(aes(y = scaled_firms, linetype = "Number of firms")) +
    theme_minimal() + scale_y_continuous(labels=label_comma(accuracy = 0.01)) +
    ylim(0,2.1) +
    scale_linetype_manual(values = c("dashed", "dotted", "solid")) + 
    scale_y_continuous(sec.axis = sec_axis(~ .* scale_factor / 2), name = "firms") + 
    ggtitle(country_name) +
    theme(text = element_text(family="serif"), axis.text.x = element_text(angle = 45),
          axis.title = element_blank(), legend.title = element_blank(), 
          legend.position = "none", title = element_text(size = 8)) 
  if (!yaxis_left)
    p <- p + theme(axis.text.y.left = element_blank())
  if (!xaxis)
    p <- p + theme(axis.text.x = element_blank())
  p
}

# create plots for each country and store them in a list
plot_list <- lapply(plotdat2, my_plot)

# get legend from one plot and use for all
plot_legend <- get_plot_component(plot_list[[1]] + theme(legend.position = "bottom"), 'guide-box-bottom', return_all = TRUE)
# arrange plots in two pages and save as separate files
plots1 <- plot_grid(plotlist = plot_list[1:16], ncol = 4)
plots1 <- plot_grid(plots1, plot_legend, ncol = 1, rel_heights = c(4, .1) )
plots2 <- plot_grid(plotlist = plot_list[17:31], ncol = 4)
plots2 <- plot_grid(plots2, plot_legend, ncol = 1, rel_heights = c(4, .1) )
ggsave("capfirms1.jpg", plots1, width = 6.5, height = 7.5, units = "in")
ggsave("capfirms2.jpg", plots2, width = 6.5, height = 7.5, units = "in")
