library(readxl) # version 1.4.3 (read_excel)
library(plm) # version 2.6.5 (plm)
library(clusterSEs) #version 2.6.5 (cluster.wild.plm)
library(fwildclusterboot) # version 0.13.0 (boottest: can be used to bootstrap standard errors)
library(lfe) # version 3.0-0 (felm)
library(stargazer) # version 5.2.3 (stargazer)
library(texreg) # version 1.39.3 (htmlreg)
library(fixest) # version 0.12.0 (feols - for Sun-Abraham estimator)
library(ggplot2) #version 3.5.1 (ggplot)
library(DescTools) # version 0.99.54 (Dummy)
library(cowplot) # version 1.1.3

rm(list=ls())
### Preparing data and variables ###
pan <- read_excel("panel_7_2023.xlsx") # read in panel (created by build_panel)
EU_15 <- c("Austria", "Belgium", "Denmark", "Finland", "France", "Germany", "Greece", "Ireland", "Italy", "Luxembourg",
           "Netherlands", "Portugal", "Spain", "Sweden")

# some variables are  read in as character but needed as numeric
pan$corptax <- as.numeric(pan$corptax)
pan$rule_of_law <- as.numeric(pan$rule_of_law)

# selecting 1995-2020
pan3 <- subset(pan, year > 1994)
pan3 <- subset(pan3, year < 2021)

#Treatment variables (needed for DiD regressions below (Table 9)
pan3$treat_I <- pan3$Reform == "I"
pan3$treat_II <- pan3$Reform == "II"
pan3$treat_I_II <- pan3$Reform %in% c("I", "II")

# Create variables showing when the first year for a particular
# country had a specific reform 
# first_I: first year when country had Level I
# first_II: first year when country did not have Level III (i.e., it either had Level II or Level I)
first_I <- sapply(unique(pan3$country), function(X) which(pan3$country == X & pan3$Reform == "I")[1])
first_II <- sapply(unique(pan3$country), function(X) which(pan3$country == X & pan3$Reform != "III")[1])
pan3$first_I <- pan3$year[first_I[pan3$country]]
pan3$first_II <- pan3$year[first_II[pan3$country]]

# year of treatment needs to be marked as "infinite" 
pan3$first_I[is.na(pan3$first_I)] <- Inf
pan3$first_II[is.na(pan3$first_II)] <- Inf

# Table 9: DiD regressions
# I run each regression in parallel using plm and felm (from the lfe package) in
# parallel (same results on point estimates). The reason for felm is solely 
# that I want to create the more nuanced R2 results from texreg (which it
# does not produce with plm. I use plm because it works with clusterSEs for 
# inference.
## (1) "Abolish High Minimum Capital (III)
## defining treatment & control groups:
## All observations: Countries that ever had level III (high minimum capital)
## Treatment group: Levels I or II; Control group: III
pan3_did1 <- subset(pan3, country %in% names(which(table(pan3$country, pan3$Reform)[, "III"] > 0)))
pan3_did1$treat <- pan3_did1$treat_I_II
pan3_did1$country <- factor(pan3_did1$country)
regDiD1.plm <- plm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros
                   + treat + as.numeric(year):country, model = "within", effect = "twoways",
                   index = c("country", "year"), data = pan3_did1)
regDiD1.lfe <- felm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros
                    + treat + as.numeric(year):country | country + year | 0 | country,  data = pan3_did1)
# note that the warning about the Variance-covariance matrix does not
# matter because we bootstrap.
# use clusterSEs to compute wild bootstrap clustered confidence intervals 
cD1_WB.plm <- cluster.wild.plm(regDiD1.plm, pan3_did1, seed = 1234, boot.reps = 9999, cluster = "group")

## (2) "Abolish Minimum Capital (II or III)
## All observations: Countries that ever had II or III; 
## Treatment: Level I; Control: II or III
pan3_did2 <- subset(pan3, country %in% names(which(rowSums(table(pan3$country, pan3$Reform)[, c("II", "III")]) > 0)))
pan3_did2$treat <- pan3_did2$treat_I
pan3_did2$country <- factor(pan3_did2$country)
regDiD2.plm <- plm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros
                   + treat + as.numeric(year):country, model = "within", effect = "twoways",
                   index = c("country", "year"), data = pan3_did2)
regDiD2.lfe <- felm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros
                    + treat + as.numeric(year):country | country + year | 0 | country,  data = pan3_did2)
cD2_WB.plm <- cluster.wild.plm(regDiD2.plm, pan3_did2, seed = 1234, boot.reps = 9999, cluster = "group")

## (3) "Move from High to Non-Trivial Minimum Capital (III to II)
## All observations: Countries that ever had Level III
## Treatment: Level II; Control : III
pan3_did3 <- pan3_did1
pan3_did3$treat <- pan3_did3$treat_II
pan3_did3 <- pan3_did3[pan3_did3$Reform != "I", ]
regDiD3.plm <- plm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros
                   + treat + as.numeric(year):country, model = "within", effect = "twoways",
                   index = c("country", "year"), data = pan3_did3)
regDiD3.lfe <- felm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros
                    + treat + as.numeric(year):country | country + year | 0 | country,  data = pan3_did3)
cD3_WB.plm <- cluster.wild.plm(regDiD3.plm, pan3_did3, seed = 1234, boot.reps = 9999, cluster = "group")

## (4) "Abolish Non-Trivial Minimum Capital (Level II to Level I)
## All observations: Countries that ever had Level II
## Treatment: Level I; Control: Level II
pan3_did4 <- subset(pan3, country %in% names(which(table(pan3$country, pan3$Reform)[, "II"] > 0)))
pan3_did4$treat <- pan3_did4$treat_I
pan3_did4$country <- factor(pan3_did4$country) # reduce number of countries to sample to avoid problems
pan3_did4 <- pan3_did4[pan3_did4$Reform %in% c("I", "II"), ]
regDiD4.plm <- plm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros
                   + treat + as.numeric(year):country, model = "within", effect = "twoways",
                   index = c("country", "year"), data = pan3_did4)
regDiD4.lfe <- felm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros
                    + treat + as.numeric(year):country | country + year | 0 | country,  data = pan3_did4)
cD4_WB.plm <- cluster.wild.plm(regDiD4.plm, pan3_did4, seed = 1234, boot.reps = 9999, cluster = "group")

## create output for paper (Table 9)
mods <- list(regDiD1.plm, regDiD2.plm, regDiD3.plm, regDiD4.plm)
stargazer(mods, ci = TRUE, 
          ci.custom = list(cD1_WB.plm$ci, cD2_WB.plm$ci, cD3_WB.plm$ci, cD4_WB.plm$ci),
          p = list(cD1_WB.plm$p.values, cD2_WB.plm$p.values, cD3_WB.plm$p.values, cD4_WB.plm$p.values),
          column.labels = c("(1) Abolish High Minimum Capital", "(2) Abolish Minimum Capital",
                            "(3) Reduce High Minimum Capital to Non-Trivial Amount", 
                            "(4) Abolish Non-Trivial Minimum Capital"),
          model.numbers = FALSE,
          omit = "as.numeric",
          # order = c(1:3, 61:62),
          covariate.labels = c("log(GDP)", "log(migrants)", "log(total no. firms)",
                               "Treatment", "EU/EEA member * Centros"),
          title = "Table 9: Dependent variable: log(number of firms)",
          dep.var.caption = "", dep.var.labels = "",
          single.row = TRUE,
          type = "text",
          out = "TableDiD_stargazer_2023.html"
)
rm(mods)

##create the same output with textreg package, but only to get their 
##various versions of R2 (I do not use these standard errors in the paper)
htmlreg(list(regDiD1.lfe, regDiD2.lfe, regDiD3.lfe, regDiD4.lfe),
        custom.header = list("(1) Abolish High Minimum Capital"=1, 
                             "(2) Abolish Minimum Capital"=2,
                             "(3) Reduce High Minimum Capital to Non-Trivial Amount"=3, 
                             "(4) Abolish Non-Trivial Minimum Capital"=4),
        digits = 3,
        omit.coef = "as.numeric",
        stars = c(0.01, 0.05, 0.1),
        single.row = TRUE,
        file = "TableDiD_texreg_2023.html"
)

# The following part produces the Sun & Abraham alternative DiD estimator.
# It is reported in Section 4.4. and Table 10
# The four models are analogous to the "normal" DiD models above.
rsa1 <- feols(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros + sunab(first_II, year) | country + year, data = pan3_did1)
rsa2 <- feols(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros + sunab(first_I, year) | country + year, data = pan3_did2)
rsa3 <- feols(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros + sunab(first_II, year) | country + year, data = pan3_did3)
rsa4 <- feols(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros + sunab(first_I, year) | country + year, data = pan3_did4)

## Output
htmlreg(list(rsa1, rsa2, rsa3, rsa4), 
        custom.model.names = c("(1) Abolish High Minimum Capital", "(2) Abolish Minimum Capital",
                               "(3) Reduce High Minimum Capital to Non-Trivial Amount", 
                               "(4) Abolish Non-Trivial Minimum Capital"),
        digits = 3,
        omit.coef = paste(unlist(lapply(c(-25:-11, 11:16), function(X) paste0("year::", X))), collapse = "|"),
        custom.coef.names = c("log(GDP)", "log(migrants)", "log(total no. firms)", 
                              sapply(c(-10:-2, 0:10), function(X) paste("Year", X)), "EU/EEA * Centros"),
        reorder.coef = c(1:3, 24, 4:23),
        stars = c(0.01, 0.05, 0.1),
        caption = "Table 10: Sun-Abraham models for heterogeneous treatment effects",
        caption.above = TRUE,
        single.row = TRUE,
        file = "Sunab_11_2023.html"
)

# This creates a graph showing the confidence interval of model 1 above (Figure 2)
##create a data.frame with point estimates and confidence intervals for relative years
gtab3 <- data.frame(year = c(-10:-2, 0:10), coef = coef(rsa1)[13:32], lower = confint(rsa1)[13:32,1], upper = confint(rsa1)[13:32,2])
gtab3 <- rbind(gtab3, c(-1, 0, 0, 0))
## convert into long format
gtab4 <- data.frame(yearrel = rep(gtab3$year, 3), type = as.factor(rep(c("pointtest", "lower", "upper" ), each = 21)),
                    value = as.vector(as.matrix(gtab3[, c("coef", "lower", "upper")])))
## graph it
ggplot(gtab4, aes(x = yearrel, y = value, color = type)) + geom_line() + theme_minimal() +
  scale_color_manual(values = c("lower" = "darkgray", "pointtest" = "black", "upper" = "darkgray"), breaks = c("lower", "pointtest"), 
                     labels = c("95% confidence interval", "point estimate"), name = element_blank()) +
  geom_vline(xintercept = -1, linetype = "dashed", color = "gray") +
  theme(legend.position = "bottom", text = element_text(family="serif")) +
  ylab("ln(number of cross-incorporations)")  + xlab("year")
ggsave("sunab_error_bars2-11-2023.jpg", width = 5, height = 3)

# The purpose of this part is to run a regression and create a graph showing the DiD impact 
# following Autor, Dononue & Schwab 2006 (Figure 1 in Part 4.3)

## This function creates a plot from a regression model that has years -10 to +10 relative 
## to reform as variables
myDiDplot <- function(my_model, my_title = "",var_label = "value", no.x.label = FALSE, no.y.label = FALSE, my_ylim = NULL) {
  ## create long format data frame with point estimates and confidence intervals
  gdat <- data.frame(yearrel = rep(-10:10, 3), type = as.factor(rep(c("pointtest", "lower", "upper" ), each = 21)),
                     value = c(pointest = coef(my_model), as.vector(confint(my_model, level = .9))))
  ## create plot
  myplot <- ggplot(gdat, aes(x = yearrel, y = value, color = type)) + geom_line() + theme_minimal() +
    scale_color_manual(values = c("pointtest" = "black", "lower" = "darkgray", "upper" = "darkgray"), breaks = c("lower", "pointtest"), 
                       labels = c("90% confidence interval", "point estimate"), name = element_blank()) +
    geom_vline(xintercept = -1, linetype = "dashed", color = "gray") +
    theme(legend.position = "bottom", text = element_text(family="serif"))
  ## add limits if they were provided in the function call
  if (!is.null(my_ylim)) {
    myplot = myplot + ylim(my_ylim[1], my_ylim[2])
  }
  ## x and y labels
  if (no.x.label) {
    myplot <- myplot + xlab("")
  } else
  {
    myplot <- myplot + xlab("years relative to reform")
  }
  if (no.y.label) {
    myplot <- myplot + ylab("")
  } else
  {
    myplot <- myplot + ylab(var_label)
  }
  myplot <- myplot + ggtitle(my_title)
  return(myplot)
}

pan4 <- pan3[, c("country", "year", "lPD")]
# creating dummies for the time from/to 
pan4$offset_ref1 <- pan3$year - as.numeric(pan3$first_I)
pan4$offset_ref2 <- pan3$year - as.numeric(pan3$first_II)
pan4 <- cbind(pan4, Dummy(pan4$offset_ref1), Dummy(pan4$offset_ref2))
names(pan4)[6:56] <- paste0("rI", names(pan4)[6:56])
names(pan4)[57:100] <- paste0("rII", names(pan4)[57:100])
names(pan4) <- gsub("-", "_", names(pan4))
pan4$country <- as.factor(pan4$country)

# regression for Level I
relregI1 <- felm(lPD ~ rI_10 + rI_9 + rI_8 + rI_7 + rI_6 + rI_5 + rI_4 + rI_3 + rI_2 + rI_1 + 
                   rI0 + rI1 + rI2 + rI3 + rI4 + rI5 + rI6 + rI7 + rI8 + rI9 + rI10 | country + year | 0 | country, data = pan4)
#plot
DiDplot1 <- myDiDplot(relregI1, my_title = "Elimination of non-trivial capital", var_label = "ln(number of cross-incorporations)", no.x.label = TRUE, no.y.label = TRUE, my_ylim = c(-1.5, 1.25))

# regression for Level II
relregII1 <- felm(lPD ~ rII_10 + rII_9 + rII_8 + rII_7 + rII_6 + rII_5 + rII_4 + rII_3 + rII_2 + rII_1 + 
                    rII0 + rII1 + rII2 + rII3 + rII4 + rII5 + rII6 + rII7 + rII8 + rII9 + rII10 | country + year | 0 | country, data = pan4)
#plot
DiDplot2 <- myDiDplot(relregII1, my_title = "Elimination of high minimum capital", 
                      "ln(number of cross-incorporations)", no.x.label = TRUE, no.y.label = FALSE, my_ylim = c(-1.5, 1.25))

# combine plots and save them
plot_legend <- get_legend(DiDplot1)
plots_comb <- plot_grid(plotlist = list(DiDplot2 + theme(legend.position = "none"), DiDplot1  + 
                                          theme(legend.position = "none")), nrow = 1) 
plots_final <- plot_grid(plots_comb, plot_legend, nrow = 3, ncol = 1, rel_heights = c(4, .1))
ggsave("eventplot.jpg", plots_final, width = 7, height = 5)

# additional est for Parallel trends assumption. Appendix, Table 2
# Parallel trends assumption: compare only countries that had high legal capital!
pan5 <- pan4[pan4$country %in% c("Austria", "Belgium", "Denmark", "Germany",
                                 "Hungary", "Netherlands", "Poland"),
             c("country", "year", "lPD", "rII_5", "rII_4", "rII_3", "rII_2", "rII_1", 
               "rII0", "rII1", "rII2", "rII3", "rII4", "rII5")]
# Parallel trends assumption for Ref 1: compare only countries that ever had 
# non-trivial minimum capital
pan6 <- pan4[!(pan4$country %in% c("Cyprus", "Ireland", "Malta", "Romania")),
             c("country", "year", "lPD", "rI_5", "rI_4", "rI_3", "rI_2", "rI_1", 
               "rI0", "rI1", "rI2", "rI3", "rI4", "rI5")]
# prep panels for nice table presentation
names(pan5)[4:14] <- paste0("event", c(paste0("_", 5:1), 0:5))
names(pan6)[4:14] <- paste0("event", c(paste0("_", 5:1), 0:5))
# remove superfluous factor levels
pan5$country <- factor(pan5$country)
pan6$country <- factor(pan6$country)

# The boottestdf function takes an lm or felm regression model. 
# It loops through all the independent variable sand runs wild bootstrap
# clustering for each of them. Results are reported in a data.frame.
## my_model: input regression model (lm or felm)
## my_type: type of bootstrap (passed on to bootstrap function of fwildclusterboot)
boottestdf <- function(my_model, my_type = "rademacher"){
  stopifnot(require(fwildclusterboot))
  stopifnot(require(dqrng))
  my_list <- lapply(names(coef(my_model)), function(my_coef){ # loop throug coefficients
    print(my_coef)
    if (is.na(coef(my_model)[my_coef])) { # if coefficient is NA -> return NA
      my_df <- data.frame(estimate = NA, ci.lower = NA, ci.upper = NA, 
                          t_stat = NA, p_val = NA, row.names = my_coef)
    } else { # otherwise run boottest, clustering by country
      dqset.seed(1234)
      my_boottest <-  boottest(my_model, param = my_coef, B = 9999, clustid = "country", type = my_type)
      my_df <- data.frame(estimate = my_boottest$point_estimate, # results are reported for each variable in a row of a data.frame
                          ci.lower = my_boottest$conf_int[[1]], 
                          ci.upper = my_boottest$conf_int[[2]],
                          t_stat = my_boottest$t_stat, p_val = my_boottest$p_val,
                          row.names = my_coef)
    }
    return(my_df)
  })
  as.data.frame(do.call(rbind, my_list)) # since lapply returns a list, the lines have to be merged in
}

set.seed(12345)
relregI2 <- felm(lPD ~ event_5 + event_4 + event_3 + event_2 + event_1 + 
                   event0 + event1 + event2 + event3 + event4 + event5 | country + year| 0 | country, data = pan6)
wcl_rrI2 <- boottestdf(relregI2)
relregII2 <- felm(lPD ~ event_5 + event_4 + event_3 + event_2 + event_1 + 
                    event0 + event1 + event2 + event3 + event4 + event5 | country + year| 0 | country, data = pan5)
wcl_rrII2 <- boottestdf(relregII2, "webb")

## Output
stargazer(list(relregI2, relregII2), ci = TRUE, 
          ci.custom = list(wcl_rrI2[, c("ci.lower", "ci.upper")],
                           wcl_rrII2[, c("ci.lower", "ci.upper")]),
          p = list(wcl_rrI2$p_val, wcl_rrII2$p_val),
          column.labels = c("Elimination of non-trivial minimum capital", "Elimination of high minimum capital"),
          covariate.labels = c(paste0("Year - ", 5:1), "Year 0", paste0("Year + ", 1:5)),
          title = "Table X: Testing for Pretreatment trends",
          dep.var.caption = "dependent variable: log(number of firms)", dep.var.labels = "",
          single.row = TRUE,
          type = "html", out = "parallel_trends_appendix.html")
