library(readxl) # version 1.4.3 (read_excel)
library(plm) # version 2.6.5 (used for lag and lead functions:
## I am not using plm for regressions because it does not interoperate
## with the libraries that I use for wil bootstrap clustering.)
library(fwildclusterboot) # version 0.13.0 (boottest: used to bootstrap standard errors)
library(dqrng) # version 0.4.0 (random seeds, needed by fwildclusterboot)
library(lfe) # version 3.0-0 (felm)
library(stargazer) # version 5.2.3 (stargazer)
library(texreg) # version 1.39.3 (htmlreg)

rm(list=ls())
### Preparing data and variables ###
pan <- read_excel("panel_7_2023.xlsx") # read in panel (created by build_panel)
EU_15 <- c("Austria", "Belgium", "Denmark", "Finland", "France", "Germany", "Greece", "Ireland", "Italy", "Luxembourg",
           "Netherlands", "Portugal", "Spain", "Sweden")
# some variables are  read in as character but needed as numeric
pan$corptax <- as.numeric(pan$corptax)
pan$rule_of_law <- as.numeric(pan$rule_of_law)

# use the plm format for the data
dp <- pdata.frame(subset(pan, year %in% 1995:2020), index = c("country", "year"))

# The boottestdf function takes an lm or felm regression model. 
# It loops through all the independent variable sand runs wild bootstrap
# clustering for each of them. Results are reported in a data.frame.
## my_model: input regression model (lm or felm)
## my_type: type of bootstrap (passed on to bootstrap function of fwildclusterboot)
boottestdf <- function(my_model, my_type = "rademacher"){
  stopifnot(require(fwildclusterboot))
  stopifnot(require(dqrng))
  my_list <- lapply(names(coef(my_model)), function(my_coef){ # loop throug coefficients
    print(my_coef)
    if (is.na(coef(my_model)[my_coef])) { # if coefficient is NA -> return NA
      my_df <- data.frame(estimate = NA, ci.lower = NA, ci.upper = NA, 
                          t_stat = NA, p_val = NA, row.names = my_coef)
    } else { # otherwise run boottest, clustering by country
      dqset.seed(1234)
      my_boottest <-  boottest(my_model, param = my_coef, B = 9999, clustid = "country", type = my_type)
      my_df <- data.frame(estimate = my_boottest$point_estimate, # results are reported for each variable in a row of a data.frame
                          ci.lower = my_boottest$conf_int[[1]], 
                          ci.upper = my_boottest$conf_int[[2]],
                          t_stat = my_boottest$t_stat, p_val = my_boottest$p_val,
                          row.names = my_coef)
    }
    return(my_df)
  })
  as.data.frame(do.call(rbind, my_list)) # since lapply returns a list, the lines have to be merged in
}

# minimum capital regressions (Table 6)
## I use the felm function from lfe for fixed effects models because
## it appears to work best with wild bootstrap clustering (fwildclusterboot)
## The felm models produce warnings relating to the variance-coveriance matrix (which I do not get when using lm or plm).
## These warnings do not matter for the output because I a report only the 
## point estimates from these models and bootstrapped confidence intervals
## from fwildclusterboot.
set.seed(123456)
regCap1 <- felm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros
                + l.mincap.PP + (as.numeric(year):country) | country + year |
                  0 | country + year, data = dp)
cC1_WB <- boottestdf(regCap1)
regCap2 <- update(regCap1, . ~ . + corptax)
cC2_WB <- boottestdf(regCap2)
regCap3 <- update(regCap2, . ~ . + rule_of_law)
cC3_WB <- boottestdf(regCap3)
regCap4 <- update(regCap1, . ~ . - (as.numeric(year):country)+ EU_EEA:Centros*l.mincap.PP)
cC4_WB <- boottestdf(regCap4)
regCap5 <- update(regCap1, . ~ . + EU_EEA:Centros*l.mincap.PP)
cC5_WB <- boottestdf(regCap5)

#I create tables using both the stargazer and the texreg package.
#For the paper, I use primarily the stargazer format (whic I like
#better), but I add the different versions of R2 from texreg.

htmlreg(list(regCap1, regCap2, regCap3, regCap4, regCap5), ci.force = TRUE,
        override.ci.low = list(cC1_WB$ci.lower, cC2_WB$ci.lower, cC3_WB$ci.lower, cC4_WB$ci.lower, cC5_WB$ci.lower),
        override.ci.up = list(cC1_WB$ci.upper, cC2_WB$ci.upper, cC3_WB$ci.upper, cC4_WB$ci.upper, cC5_WB$ci.upper),
        override.pvalues = list(as.vector(cC1_WB$p_val), as.vector(cC2_WB$p_val), as.vector(cC3_WB$p_val), as.vector(cC4_WB$p_val), as.vector(cC5_WB$p_val)),
        custom.model.names = c("(1) 1995-2020", "(2) 1995-2020", "(3) 1996-2020", "(4) 1995-2020", "(5) 1995-2020"),
        digits = 3,
        omit.coef = "as.numeric",
        custom.coef.names = c("log(GDP)", "log(migrants)", "log(total no. firms)",
                              "log(Minimum capital / GDP-PPP)",
                              "EU/EEA member * Centros", "Corporate tax", "Rule of Law",
                              "EU/EEA member * Centros * log(Minimum capital / GDP-PPP)"),
        reorder.coef = c(1:3, 6:7, 4, 5, 8),
        # ci.test = NA,
        stars = c(0.01, 0.05, 0.1),
        caption = "Table 6: Dependent variable: log(number of firms)",
        caption.above = TRUE,
        file = "Table6_texreg.html"
)

stargazer(list(regCap1, regCap2, regCap3, regCap4, regCap5), ci = TRUE, 
          ci.custom = list(cC1_WB[, c("ci.lower", "ci.upper")],
                           cC2_WB[, c("ci.lower", "ci.upper")],
                           cC3_WB[, c("ci.lower", "ci.upper")],
                           cC4_WB[, c("ci.lower", "ci.upper")],
                           cC5_WB[, c("ci.lower", "ci.upper")]),
          p = list(cC1_WB$p_val, cC2_WB$p_val, NULL, #cC3_WB$p_val, 
                   cC4_WB$p_val, cC5_WB$p_val),
          column.labels = c("(1) 1995-2020", "(2) 1995-2020", "(3) 1996-2020", "(4) 1995-2020", "(5) 1995-2020"),
          model.numbers = FALSE,
          omit = "as.numeric",
          order = c(1:3,5:6,4,7:8),
          covariate.labels = c("log(GDP)", "log(migrants)", "log(total no. firms)",
                               "Corporate tax", "Rule of Law", 
                               "log(Minimum capital / GDP-PPP)",
                               "EU/EEA member * Centros",
                               "EU/EEA member * Centros * log(Minimum capital / GDP-PPP)"),
          title = "Table 6: Dependent variable: log(number of firms)",
          dep.var.caption = "", dep.var.labels = "",
          single.row = TRUE,
          type = "html", out = "Table6_stargazer.html")

## Analogous minimum pay-in regressions (Table 12 in the Appendix)
set.seed(123456)
regPay1 <- felm(lPD ~ l.GDP + l.migrants + l.totalfirms +
                  EU_EEA:Centros  + l.minpayin.PP + (as.numeric(year):country)| country + year |
                  0 | country + year, data = dp)
cP1_WB <- boottestdf(regPay1)
regPay2 <- update(regPay1, . ~ . + corptax)
cP2_WB <- boottestdf(regPay2)
regPay3 <- update(regPay2, . ~ . + rule_of_law)
cP3_WB <- boottestdf(regPay3)
regPay4 <- update(regPay1, . ~ . - (as.numeric(year):country)+ EU_EEA:Centros*l.minpayin.PP)
cP4_WB <- boottestdf(regPay4)
regPay5 <- update(regPay1, . ~ . + EU_EEA:Centros*l.minpayin.PP)
cP5_WB <- boottestdf(regPay5)

htmlreg(list(regPay1, regPay2, regPay3, regPay4, regPay5), ci.force = TRUE,
        override.ci.low = list(cP1_WB$ci.lower, cP2_WB$ci.lower, cP3_WB$ci.lower, cP4_WB$ci.lower, cP5_WB$ci.lower),
        override.ci.up = list(cP1_WB$ci.upper, cP2_WB$ci.upper, cP3_WB$ci.upper, cP4_WB$ci.upper, cP5_WB$ci.upper),
        override.pvalues = list(as.vector(cP1_WB$p_val), as.vector(cP2_WB$p_val), as.vector(cP3_WB$p_val), as.vector(cP4_WB$p_val), as.vector(cP5_WB$p_val)),
        custom.model.names = c("(1) 1995-2020", "(2) 1995-2020", "(3) 1996-2020", "(4) 1995-2020", "(5) 1995-2020"),
        digits = 3,
        omit.coef = "as.numeric",
        custom.coef.names = c("log(GDP)", "log(migrants)", "log(total no. firms)",
                              "log(Minimum pay-in / GDP-PPP)",
                              "EU/EEA member * Centros", "Corporate tax", "Rule of Law",
                              "EU/EEA member * Centros * log(Minimum capital / GDP-PPP)"),
        reorder.coef = c(1:3, 6:7, 4, 5, 8),
        stars = c(0.01, 0.05, 0.1),
        caption = "Table 12: Effects of Capital pay-in requireent on log(number of firms)",
        caption.above = TRUE,
        file = "Table12_texreg.html"
)

stargazer(list(regPay1, regPay2, regPay3, regPay4, regPay5), ci = TRUE, 
          ci.custom = list(cP1_WB[, c("ci.lower", "ci.upper")],
                           cP2_WB[, c("ci.lower", "ci.upper")],
                           cP3_WB[, c("ci.lower", "ci.upper")],
                           cP4_WB[, c("ci.lower", "ci.upper")],
                           cP5_WB[, c("ci.lower", "ci.upper")]),
          p = list(cP1_WB$p_val, cP2_WB$p_val, cP3_WB$p_val, cP4_WB$p_val, cP5_WB$p_val),
          column.labels = c("(1) 1995-2020", "(2) 1995-2020", "(3) 1996-2020", "(4) 1995-2020", "(5) 1995-2016"),
          model.numbers = FALSE,
          omit = "as.numeric",
          order = c(1:3,5:6,4,7:8),
          covariate.labels = c("log(GDP)", "log(migrants)", "log(total no. firms)",
                               "Corporate tax", "Rule of Law", 
                               "log(Minimum pay-in / GDP-PPP)",
                               "EU/EEA member * Centros",
                               "EU/EEA member * Centros * log(Minimum capital / GDP-PPP)"),
          title = "Table 12: Effects of Capital pay-in requireent on log(number of firms)",
          dep.var.caption = "", dep.var.labels = "",
          single.row = TRUE,
          type = "html", out = "Table12_stargazer.html")

# Appendix, Table 11: 
## mincap regressions slowly building up from pooled model to full model
set.seed(123456)
regCap0A <- felm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA * Centros + l.mincap.PP | 0 | 0 | country + year , 
                 data = dp)
summary(regCap0A, robust =T)
cC0A_WB <- boottestdf(regCap0A)
regCap0B <- felm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA * Centros + l.mincap.PP | country | 0 | country, 
                 data = dp)
cC0B_WB <- boottestdf(regCap0B)
regCap0C <- felm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA * Centros + l.mincap.PP + as.numeric(year) | country | 0 | country, 
                 data = dp)
cC0C_WB <- boottestdf(regCap0C)
regCap0D <- felm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA * Centros + l.mincap.PP | year | 0 | country, 
                 data = dp)
cC0D_WB <- boottestdf(regCap0D)
regCap0E <- felm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA * Centros + l.mincap.PP | country + year | 0 | country, 
                 data = dp)
cC0E_WB <- boottestdf(regCap0E)

htmlreg(list(regCap0A, regCap0B, regCap0C, regCap0D, regCap0E, regCap1),
        ci.force = TRUE,
        override.ci.low = list(cC0A_WB$ci.lower, cC0B_WB$ci.lower, cC0C_WB$ci.lower, 
                               cC0D_WB$ci.lower, cC0E_WB$ci.lower, cC1_WB$ci.lower),
        override.ci.up = list(cC0A_WB$ci.upper, cC0B_WB$ci.upper, cC0C_WB$ci.upper, 
                              cC0D_WB$ci.upper, cC0E_WB$ci.upper, cC1_WB$ci.upper),
        override.pvalues = list(cC0A_WB$p_val, cC0B_WB$p_val, cC0C_WB$p_val,
                                cC0D_WB$p_val, cC0E_WB$p_val, cC1_WB$p_val),
        custom.model.names = c("Pooled model", "Country FE", "Country FE & Global Trend", "Year FE", "Country & Year FE",
                               "Table 6, Model 1"),
        digits = 3,
        omit.coef = "country",
        custom.coef.names = c("Constant", "log(GDP)", "log(migrants)", "log(total no. firms)",
                              "EU/EEA member", "Centros", "log(Minimum capital / GDP-PPP)",
                              "EU/EEA member post-Centros", "Year trend"),
        reorder.coef = c(1:4,7,5:6, 8:9),
        stars = c(0.01, 0.05, 0.1),
        caption = "Table 11: Effect of minimum capital on log(number of firms)",
        caption.above = TRUE,
        file = "Table11_texreg.html"
)

stargazer(list(regCap0A, regCap0B, regCap0C, regCap0D, regCap0E, regCap1), 
          ci = TRUE, 
          ci.custom = list(cC0A_WB[, c("ci.lower", "ci.upper")],
                           cC0B_WB[, c("ci.lower", "ci.upper")],
                           cC0C_WB[, c("ci.lower", "ci.upper")],
                           cC0D_WB[, c("ci.lower", "ci.upper")],
                           cC0E_WB[, c("ci.lower", "ci.upper")],
                           cC1_WB[, c("ci.lower", "ci.upper")]),
          p = list(cC0A_WB$p_val, cC0B_WB$p_val, cC0C_WB$p_val, cC0D_WB$p_val, cC0E_WB$p_val, cC1_WB$p_val),
          column.labels = c("Pooled model", "Country FE", "Country FE and Global Trend", "Year FE", "Country and Year FE",
                            "Table 6, Model 1"),
          model.numbers = FALSE,
          omit = "country",
          intercept.top = TRUE, intercept.bottom = FALSE,
          order = c(1:4,7,5:6,9,8),
          covariate.labels = c("Constant", "log(GDP)", "log(migrants)", "log(total no. firms)",
                               "log(Minimum capital / GDP-PPP)",
                               "EU/EEA member", "Centros",
                               "EU/EEA member * Centros", "Year trend"),
          title = "Table 11: Effect of minimum capital on log(number of firms)",
          dep.var.caption = "", dep.var.labels = "",
          single.row = TRUE,
          type = "html", out = "Table11_stargazer.html")


# Table 14: Key models from Tables 6 and 12 but using GDP as weight.
## I run these regressions with both lm and felm (point estimates are the same)
## (lm because fwildclusterboot seems to have some issues with some of 
## the weighted regressions in felm; I use felm with textreg only to 
## produce the multiple versions of R2)

## minimum capital
set.seed(123456)
regCap1.w <- lm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros + l.mincap.PP + 
                  (as.numeric(year):country) + country + year, data = dp, weights = dp$GDP)
cC1.w <- boottestdf(regCap1.w)
regCap1.w.felm <- update(regCap1, weights = dp$GDP)
regCap4.w <- lm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros + l.mincap.PP + 
                  EU_EEA:Centros:l.mincap.PP + country + year, data = dp, weights = dp$GDP)
cC4.w <- boottestdf(regCap4.w)
regCap4.w.felm <- update(regCap4, weights = dp$GDP)
regCap5.w <- lm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros + l.mincap.PP + 
                  (as.numeric(year):country) + EU_EEA:Centros:l.mincap.PP + country + year, data = dp, weights = dp$GDP)
cC5.w <- boottestdf(regCap5.w) 
regCap5.w.felm <- update(regCap5, weights = dp$GDP)

## minimum pay-in ##
set.seed(123456)
regPay1.w <-lm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros + l.minpayin.PP + 
                 (as.numeric(year):country) + country + year, data = dp, weights = dp$GDP)
cP1.w <- boottestdf(regPay1.w)
regPay1.w.felm <- update(regPay1, weights = dp$GDP)

regPay4.w <- lm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros + l.minpayin.PP + 
                  EU_EEA:Centros:l.minpayin.PP + country + year, data = dp, weights = dp$GDP)
cP4.w <- boottestdf(regPay4.w)
regPay4.w.felm <- update(regPay4, weights = dp$GDP)
regPay5.w <- lm(lPD ~ l.GDP + l.migrants + l.totalfirms + EU_EEA:Centros + l.minpayin.PP + 
                  (as.numeric(year):country) + EU_EEA:Centros:l.minpayin.PP + country + year, data = dp, weights = dp$GDP)
cP5.w <- boottestdf(regPay5.w)
regPay5.w.felm <- update(regPay5, weights = dp$GDP)

##Output for Table 14 (Appendix)
sglist <- list(regCap1.w, regCap4.w, regCap5.w, regPay1.w, regPay4.w, regPay5.w)
stargazer(sglist, 
          ci = TRUE, 
          ci.custom = list(cC1.w[, c("ci.lower", "ci.upper")],
                           cC4.w[, c("ci.lower", "ci.upper")],
                           cC5.w[, c("ci.lower", "ci.upper")],
                           cP1.w[, c("ci.lower", "ci.upper")],
                           cP4.w[, c("ci.lower", "ci.upper")],
                           cP5.w[, c("ci.lower", "ci.upper")]),
          p = list(cC1.w$p_val, cC4.w$p_val, cC5.w$p_val, cP1.w$p_val, cP4.w$p_val, cP5.w$p_val),
          omit = c("as.numeric", "year", "country"),
          order = c(1:3, 61, 4, 93, 5, 94),
          covariate.labels = c("log(GDP)", "log(migrants)", "log(total no. firms)",
                               "EU/EEA member * Centros",
                               "log(Minimum capital / GDP-PPP)",
                               "EU/EEA member * Centros * log(Minimum capital / GDP-PPP)",
                               "log(Minimum pay-in / GDP-PPP)",
                               "EU/EEA member * Centros * log(Minimum pay-in / GDP-PPP)"),
          single.row = TRUE,
          type = "html", out = "weightreg_stargazer.html")
rm(sglist)

## using this only for different versions of R-Squares because boottest 
## Since I don't use it, I do not bother to replace the CIs 
## with the bootstrapped ones. (The error messages/warnings are irrelevant because
## they concern Covariance Matrixes that are not reported.)
htmlreg(list(regCap1.w.felm, regCap4.w.felm, regCap5.w.felm, regPay1.w.felm, 
             regPay4.w.felm, regPay5.w.felm), 
        ci.force = TRUE, 
        custom.header = list("Weighted Table 6" = 1:3, "Weighted Table 7" = 4:6),
        custom.model.names = rep(c("Model 1", "Model 4", "Model 5"),2),
        digits = 3,
        omit.coef = "as.numeric",
        stars = c(0.01, 0.05, 0.1),
        caption = "Table 14: Weighted versions of key models in Tables 6 and 7",
        caption.above = TRUE,
        single.row = TRUE,
        file = "weightreg_texreg_felm.html"
)

# Models with lagging and leading dependent variables
dp2 <- dp
dp2$l.mincap.PP_lag1 <- lag(dp2$l.mincap.PP, 1)
dp2$l.mincap.PP_lag2 <- lag(dp2$l.mincap.PP, 2)
dp2$l.mincap.PP_lag3 <- lag(dp2$l.mincap.PP, 3)
dp2$l.mincap.PP_lead1 <- lead(dp2$l.mincap.PP, 1)
dp2$l.mincap.PP_lead2 <- lead(dp2$l.mincap.PP, 2)
dp2$l.mincap.PP_lead3 <- lead(dp2$l.mincap.PP, 3)
dp2$l.minpayin.PP_lag1 <- lag(dp2$l.minpayin.PP, 1)
dp2$l.minpayin.PP_lag2 <- lag(dp2$l.minpayin.PP, 2)
dp2$l.minpayin.PP_lag3 <- lag(dp2$l.minpayin.PP, 3)
dp2$l.minpayin.PP_lead1 <- lead(dp2$l.minpayin.PP, 1)
dp2$l.minpayin.PP_lead2 <- lead(dp2$l.minpayin.PP, 2)
dp2$l.minpayin.PP_lead3 <- lead(dp2$l.minpayin.PP, 3)

### lag models for capital ###
set.seed(123456)
regCap1.lag1 <- update(regCap1, formula = .~.+ l.mincap.PP_lag1, data = dp2)
cC1.l1 <- boottestdf(regCap1.lag1)
regCap1.lag2 <- update(regCap1.lag1, formula = .~.+ l.mincap.PP_lag2)
cC1.l2 <- boottestdf(regCap1.lag2)
regCap1.lag3 <- update(regCap1.lag2, formula = .~.+ l.mincap.PP_lag3)
cC1.l3 <- boottestdf(regCap1.lag3)

## lead models for capital
set.seed(123456)
regCap1.lead1 <- update(regCap1, formula = .~.+ l.mincap.PP_lead1, data = dp2)
cC1.ld1 <- boottestdf(regCap1.lead1)
regCap1.lead2 <- update(regCap1.lead1, formula = .~.+ l.mincap.PP_lead2)
cC1.ld2 <- boottestdf(regCap1.lead2)
regCap1.lead3 <- update(regCap1.lead2, formula = .~.+ l.mincap.PP_lead3)
cC1.ld3 <- boottestdf(regCap1.lead3)

# Table 7: lag & lead model output
htmlreg(list(regCap1, regCap1.lag1, regCap1.lag2, regCap1.lag3,
             regCap1.lead1, regCap1.lead2, regCap1.lead3), ci.force = TRUE,
        override.ci.low = list(cC1_WB$ci.lower, cC1.l1$ci.lower, cC1.l2$ci.lower,
                               cC1.l3$ci.lower, cC1.ld1$ci.lower, 
                               cC1.ld2$ci.lower, cC1.ld3$ci.lower),
        override.ci.up = list(cC1_WB$ci.upper, cC1.l1$ci.upper, cC1.l2$ci.upper,
                              cC1.l3$ci.upper, cC1.ld1$ci.upper, 
                              cC1.ld2$ci.upper, cC1.ld3$ci.upper),
        override.pvalues = list(cC1_WB$p_val, cC1.l1$p_val, cC1.l2$p_val,
                                cC1.l3$p_val, cC1.ld1$p_val, 
                                cC1.ld2$p_val, cC1.ld3$p_val),
        custom.header = list("Table 6, Model 1" = 1, "Lagged Models" = 2:4, "Leading Models" = 5:7),
        custom.model.names = rep(" ", 7),
        custom.coef.map = list("l.GDP" = "log(GDP)", "l.migrants" = "log(migrants)",
                               "l.totalfirms" = "log(total no. firms)",
                               "EU_EEA" = "EU_EEA member",
                               "EU_EEA:Centros" = "EU/EEA member * Centros",
                               "l.mincap.PP" = "log(Minimum capital / GDP-PPP)",
                               "l.mincap.PP_lag1" = "-- one-year lag",
                               "l.mincap.PP_lag2" = "-- two-year lag",
                               "l.mincap.PP_lag3" = "-- three-year lag",                               "l.mincap.PP_lead1" = "-- one-year lead",
                               "l.mincap.PP_lead2" = "-- two-year lead",
                               "l.mincap.PP_lead3" = "-- three-year lead"),
        digits = 3,
        stars = c(0.01, 0.05, 0.1),
        caption = "Table 7: Lagging and Leading  minimum capital models",
        caption.above = TRUE,
        single.row = TRUE,
        file = "laglead_mincap_texreg.html"
)

sglist <- list(regCap1, regCap1.lag1, regCap1.lag2, regCap1.lag3,
               regCap1.lead1, regCap1.lead2, regCap1.lead3)
stargazer(sglist, 
          ci = TRUE, 
          ci.custom = list(cC1_WB[, c("ci.lower", "ci.upper")],
                           cC1.l1[, c("ci.lower", "ci.upper")],
                           cC1.l2[, c("ci.lower", "ci.upper")],
                           cC1.l3[, c("ci.lower", "ci.upper")],
                           cC1.ld1[, c("ci.lower", "ci.upper")],
                           cC1.ld2[, c("ci.lower", "ci.upper")],
                           cC1.ld3[, c("ci.lower", "ci.upper")]),
          p = list(cC1_WB$p_val, cC1.l1$p_val, cC1.l3$p_val, cC1.ld1$p_val, 
                   cC1.ld1$p_val, cC1.ld2$p_val, cC1.ld3$p_val),
          omit = "as.numeric",
          order = c(1:3, 11, 4:10),
          single.row = TRUE,
          type = "html", out = "laglead_mincap_stargazer.html")
rm(sglist)

### lag models for minimum payin ###
set.seed(123456)
regPay1.lag1 <- update(regPay1, formula = .~.+ l.minpayin.PP_lag1, data = dp2)
cP1.l1 <- boottestdf(regPay1.lag1)
regPay1.lag2 <- update(regPay1.lag1, formula = .~.+ l.minpayin.PP_lag2)
cP1.l2 <- boottestdf(regPay1.lag2)
regPay1.lag3 <- update(regPay1.lag2, formula = .~.+ l.minpayin.PP_lag3)
cP1.l3 <- boottestdf(regPay1.lag3)

### lead models for minimum payin ###
set.seed(123456)
regPay1.lead1 <- update(regPay1, formula = .~.+ l.minpayin.PP_lead1, data = dp2)
cP1.ld1 <- boottestdf(regPay1.lead1)
regPay1.lead2 <- update(regPay1.lead1, formula = .~.+ l.minpayin.PP_lead2)
cP1.ld2 <- boottestdf(regPay1.lead2)
regPay1.lead3 <- update(regPay1.lead2, formula = .~.+ l.minpayin.PP_lead3)
cP1.ld3 <- boottestdf(regPay1.lead3)

# Table 13: lag & lead model output for Payin
htmlreg(list(regPay1, regPay1.lag1, regPay1.lag2, regPay1.lag3,
             regPay1.lead1, regPay1.lead2, regPay1.lead3), ci.force = TRUE,
        override.ci.low = list(cP1_WB$ci.lower, cP1.l1$ci.lower, cP1.l2$ci.lower,
                               cP1.l3$ci.lower, cP1.ld1$ci.lower, 
                               cP1.ld2$ci.lower, cP1.ld3$ci.lower),
        override.ci.up = list(cP1_WB$ci.upper, cP1.l1$ci.upper, cP1.l2$ci.upper,
                              cP1.l3$ci.upper, cP1.ld1$ci.upper, 
                              cP1.ld2$ci.upper, cP1.ld3$ci.upper),
        override.pvalues = list(cP1_WB$p_val, cP1.l1$p_val, cP1.l2$p_val,
                                cP1.l3$p_val, cP1.ld1$p_val, 
                                cP1.ld2$p_val, cP1.ld3$p_val),
        custom.header = list("Table 12, Model 1" = 1, "Lagged Models" = 2:4, "Leading Models" = 5:7),
        custom.model.names = rep(" ", 7),
        custom.coef.map = list("l.GDP" = "log(GDP)", "l.migrants" = "log(migrants)",
                               "l.totalfirms" = "log(total no. firms)",
                               "EU_EEA" = "EU_EEA member",
                               "EU_EEA:Centros" = "EU/EEA member * Centros",
                               "l.minpayin.PP" = "log(Minimum pay-in / GDP-PPP)",
                               "l.minpayin.PP_lag1" = "-- one-year lag",
                               "l.minpayin.PP_lag2" = "-- two-year lag",
                               "l.minpayin.PP_lag3" = "-- three-year lag",
                               "l.minpayin.PP_lead1" = "-- one-year lead",
                               "l.minpayin.PP_lead2" = "-- two-year lead",
                               "l.minpayin.PP_lead3" = "-- three-year lead"),
        digits = 3,
        stars = c(0.01, 0.05, 0.1),
        caption = "Table XX: Lagging and Leading  minimum pay-in models",
        caption.above = TRUE,
        single.row = TRUE,
        file = "laglead_minpayin_texreg.html"
)

sglist <- list(regPay1, regPay1.lag1, regPay1.lag2, regPay1.lag3,
               regPay1.lead1, regPay1.lead2, regPay1.lead3)
stargazer(sglist, 
          ci = TRUE, 
          ci.custom = list(cP1_WB[, c("ci.lower", "ci.upper")],
                           cP1.l1[, c("ci.lower", "ci.upper")],
                           cP1.l2[, c("ci.lower", "ci.upper")],
                           cP1.l3[, c("ci.lower", "ci.upper")],
                           cP1.ld1[, c("ci.lower", "ci.upper")],
                           cP1.ld2[, c("ci.lower", "ci.upper")],
                           cP1.ld3[, c("ci.lower", "ci.upper")]),
          p = list(cP1_WB$p_val, cP1.l1$p_val, cP1.l3$p_val, cP1.ld1$p_val, 
                   cP1.ld1$p_val, cP1.ld2$p_val, cP1.ld3$p_val),
          omit = "as.numeric",
          order = c(1:3, 11, 4:10),
          single.row = TRUE,
          type = "html", out = "laglead_minpayin_stargazer.html")
rm(sglist)